from curses import wrapper, curs_set
import time
from random import choice, randint
from abc import ABC, abstractmethod

class SimObjet(ABC):
    def __init__(self, screen):
        self.screen = screen
        self.x = self.old_x = randint(0, self.rows-1)
        self.y = self.old_y = randint(0, self.cols-1)

    @property
    def rows(self):
        return self.screen.getmaxyx()[1]

    @property
    def cols(self):
        return self.screen.getmaxyx()[0]

    @abstractmethod
    def draw(self):
        ...

class Fourmi(SimObjet):
    def __init__(self, screen):
        self.old_position = []
        self.size = 4
        super().__init__(screen)


    def walk(self):
        self.old_position.append((self.x, self.y))
        if len(self.old_position) > self.size:
            self.old_position.pop(0)
            
        x, y = self.x, self.y
        while (x,y) in self.old_position:
            x = (self.x + choice((-1, 0, 1))) % (self.rows - 1)
            y = (self.y + choice((-1, 0, 1))) % (self.cols - 1)
        self.x = x
        self.y = y

    def draw(self):
        self.screen.addch(self.y, self.x, "O")
        for x,y in self.old_position:
            self.screen.addch(y, x, "·")

@wrapper
def main(stdscr):
    stdscr.clear()
    cols, rows = stdscr.getmaxyx()
    curs_set(0)
    Fourmis = [Fourmi(stdscr) for _ in range(20)]
    while True:
        time.sleep(0.1)
        stdscr.erase()
        for F in Fourmis:
            F.walk()
            F.draw()
        stdscr.refresh()
